@@ -418,7 +418,7 @@ hipError_t hipDeviceGetAttribute(int* pi, hipDeviceAttribute_t attr, int device)
       // HIP runtime always uses SVM for host memory allocations.
       // Note: Host registered memory isn't covered by this feature
       // and still requires hipMemHostGetDevicePointer() call
-      *pi = g_devices[device]->devices()[0]->info().hostUnifiedMemory_;
+      *pi = true;
       break;
     case hipDeviceAttributeCanUseStreamWaitValue:
       // hipStreamWaitValue64() and hipStreamWaitValue32() support
@@ -442,6 +442,9 @@ hipError_t hipDeviceGetAttribute(int* pi, hipDeviceAttribute_t attr, int device)
     case hipDeviceAttributeVirtualMemoryManagementSupported:
       *pi = static_cast<int>(g_devices[device]->devices()[0]->info().virtualMemoryManagement_);
       break;
+    case hipDeviceAttributeAccessPolicyMaxWindowSize:
+      *pi = prop.accessPolicyMaxWindowSize;
+       break;
     default:
       HIP_RETURN(hipErrorInvalidValue);
   }
@@ -459,10 +462,12 @@ hipError_t hipDeviceGetByPCIBusId(int* device, const char* pciBusIdstr) {
   int pciBusID = -1;
   int pciDeviceID = -1;
   int pciDomainID = -1;
+  int pciFunction = -1;
   bool found = false;
-  if (sscanf(pciBusIdstr, "%04x:%02x:%02x", reinterpret_cast<unsigned int*>(&pciDomainID),
+  if (sscanf(pciBusIdstr, "%04x:%02x:%02x.%01x", reinterpret_cast<unsigned int*>(&pciDomainID),
              reinterpret_cast<unsigned int*>(&pciBusID),
-             reinterpret_cast<unsigned int*>(&pciDeviceID)) == 0x3) {
+             reinterpret_cast<unsigned int*>(&pciDeviceID),
+             reinterpret_cast<unsigned int*>(&pciFunction)) == 0x4) {
     int count = 0;
     HIP_RETURN_ONFAIL(ihipDeviceGetCount(&count));
     for (cl_int i = 0; i < count; i++) {
@@ -470,9 +475,11 @@ hipError_t hipDeviceGetByPCIBusId(int* device, const char* pciBusIdstr) {
       hipDeviceProp_tR0600 prop;
       HIP_RETURN_ONFAIL(ihipDeviceGet(&dev, i));
       HIP_RETURN_ONFAIL(ihipGetDeviceProperties(&prop, dev));
+      auto* deviceHandle = g_devices[dev]->devices()[0];
 
       if ((pciBusID == prop.pciBusID) && (pciDomainID == prop.pciDomainID) &&
-          (pciDeviceID == prop.pciDeviceID)) {
+          (pciDeviceID == prop.pciDeviceID) &&
+          (pciFunction == deviceHandle->info().deviceTopology_.pcie.function)) {
         *device = i;
         found = true;
         break;
@@ -507,9 +514,7 @@ hipError_t hipDeviceGetLimit(size_t* pValue, hipLimit_t limit) {
 
   switch (limit) {
     case hipLimitMallocHeapSize:
-      hipDeviceProp_tR0600 prop;
-      HIP_RETURN_ONFAIL(ihipGetDeviceProperties(&prop, ihipGetDevice()));
-      *pValue = prop.totalGlobalMem;
+      *pValue = hip::getCurrentDevice()->devices()[0]->InitialHeapSize();
       break;
     case hipLimitStackSize:
       *pValue = hip::getCurrentDevice()->devices()[0]->StackSize();
@@ -538,7 +543,12 @@ hipError_t hipDeviceGetPCIBusId(char* pciBusId, int len, int device) {
 
   hipDeviceProp_tR0600 prop;
   HIP_RETURN_ONFAIL(ihipGetDeviceProperties(&prop, device));
-  snprintf(pciBusId, len, "%04x:%02x:%02x.0", prop.pciDomainID, prop.pciBusID, prop.pciDeviceID);
+  auto* deviceHandle = g_devices[device]->devices()[0];
+  snprintf (pciBusId, len, "%04x:%02x:%02x.%01x",
+                    prop.pciDomainID,
+                    prop.pciBusID,
+                    prop.pciDeviceID,
+                    deviceHandle->info().deviceTopology_.pcie.function);
 
   HIP_RETURN(len <= 12 ? hipErrorInvalidValue : hipSuccess);
 }
@@ -606,8 +616,9 @@ hipError_t hipDeviceSetSharedMemConfig(hipSharedMemConfig config) {
 
 hipError_t hipDeviceSynchronize() {
   HIP_INIT_API(hipDeviceSynchronize);
-  constexpr bool kDoWaitForCpu = true;
-  hip::Stream::SyncAllStreams(hip::getCurrentDevice()->deviceId(), kDoWaitForCpu);
+  CHECK_SUPPORTED_DURING_CAPTURE();
+  constexpr bool kDoWaitForCpu = false;
+  hip::getCurrentDevice()->SyncAllStreams(kDoWaitForCpu);
   HIP_RETURN(hipSuccess);
 }
 
