@@ -51,9 +51,9 @@
 /*! IHIP IPC MEMORY Structure */
 #define IHIP_IPC_MEM_HANDLE_SIZE   32
 #define IHIP_IPC_MEM_RESERVED_SIZE LP64_SWITCH(20,12)
-
-extern std::once_flag g_ihipInitialized;
-
+namespace hip{
+  extern std::once_flag g_ihipInitialized;
+}
 typedef struct hipArray {
     void* data;  // FIXME: generalize this
     struct hipChannelFormatDesc desc;
@@ -69,10 +69,26 @@ typedef struct hipArray {
 }hipArray;
 
 namespace hip {
+enum MemcpyType {
+  /// Memcpy from host to host
+  hipHostToHost,
+  /// Memcpy from host to device
+  hipWriteBuffer,
+  /// Memcpy from device to host
+  hipReadBuffer,
+  /// Memcpy from device A to device A
+  /// Memcpy from pinned host buffer to device/device to pinned host buffer
+  hipCopyBuffer,
+  /// Memcpy from device A to device A, user forced to SDMA
+  hipCopyBufferSDMA,
+  /// Memcpy from device A to device B
+  hipCopyBufferP2P,
+};
 struct Graph;
 struct GraphNode;
 struct GraphExec;
 struct UserObject;
+class Stream;
 typedef struct ihipIpcMemHandle_st {
   char ipc_handle[IHIP_IPC_MEM_HANDLE_SIZE];  ///< ipc memory handle on ROCr
   size_t psize;
@@ -96,7 +112,7 @@ const char* ihipGetErrorName(hipError_t hip_error);
 #define HIP_INIT(noReturn)                                                                         \
   {                                                                                                \
     bool status = true;                                                                            \
-    std::call_once(g_ihipInitialized, hip::init, &status);                                         \
+    std::call_once(hip::g_ihipInitialized, hip::init, &status);                                         \
     if (!status && !noReturn) {                                                                    \
       HIP_RETURN(hipErrorInvalidDevice);                                                           \
     }                                                                                              \
@@ -109,7 +125,7 @@ const char* ihipGetErrorName(hipError_t hip_error);
 #define HIP_INIT_VOID()                                                                            \
   {                                                                                                \
     bool status = true;                                                                            \
-    std::call_once(g_ihipInitialized, hip::init, &status);                                         \
+    std::call_once(hip::g_ihipInitialized, hip::init, &status);                                         \
     if (hip::tls.device_ == nullptr && hip::g_devices.size() > 0) {                           \
       hip::tls.device_ = hip::g_devices[0];                                                   \
       amd::Os::setPreferredNumaNode(hip::g_devices[0]->devices()[0]->getPreferredNumaNode()); \
@@ -180,37 +196,66 @@ const char* ihipGetErrorName(hipError_t hip_error);
     }                                    \
   } while (0);
 
+// During stream capture some actions, such as a call to hipMalloc, may be unsafe and prohibited
+// during capture. It is allowed only in relaxed mode.
 #define CHECK_STREAM_CAPTURE_SUPPORTED()                                                           \
   if (hip::tls.stream_capture_mode_ == hipStreamCaptureModeThreadLocal) {                          \
     if (hip::tls.capture_streams_.size() != 0) {                                                   \
+      for (auto stream : hip::tls.capture_streams_) {                                              \
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                               \
+      }                                                                                            \
       HIP_RETURN(hipErrorStreamCaptureUnsupported);                                                \
     }                                                                                              \
   } else if (hip::tls.stream_capture_mode_ == hipStreamCaptureModeGlobal) {                        \
     if (hip::tls.capture_streams_.size() != 0) {                                                   \
+      for (auto stream : hip::tls.capture_streams_) {                                              \
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                               \
+      }                                                                                            \
       HIP_RETURN(hipErrorStreamCaptureUnsupported);                                                \
     }                                                                                              \
     if (g_captureStreams.size() != 0) {                                                            \
+      for (auto stream : g_captureStreams) {                                                       \
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                               \
+      }                                                                                            \
       HIP_RETURN(hipErrorStreamCaptureUnsupported);                                                \
     }                                                                                              \
   }
 
-// Sync APIs cannot be called when stream capture is active
+// Device sync is not supported during capture
+#define CHECK_SUPPORTED_DURING_CAPTURE()                                                           \
+  if (!g_allCapturingStreams.empty()) {                                                            \
+    for (auto stream : g_allCapturingStreams) {                                                    \
+      stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                                 \
+    }                                                                                              \
+    return hipErrorStreamCaptureUnsupported;                                                       \
+  }
+
+// Sync APIs like hipMemset, hipMemcpy etc.. cannot be called when stream capture is active
+// for all capture modes hipStreamCaptureModeGlobal, hipStreamCaptureModeThreadLocal and
+// hipStreamCaptureModeRelaxed
 #define CHECK_STREAM_CAPTURING()                                                                   \
-  if (!g_captureStreams.empty()) {                                                                 \
+  if (!g_allCapturingStreams.empty()) {                                                            \
+    for (auto stream : g_allCapturingStreams) {                                                    \
+      stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                                 \
+    }                                                                                              \
     return hipErrorStreamCaptureImplicit;                                                          \
   }
 
 #define STREAM_CAPTURE(name, stream, ...)                                                          \
   hip::getStreamPerThread(stream);                                                                 \
-  if (stream != nullptr &&                                                                         \
+  if (stream != nullptr && stream != hipStreamLegacy &&                                            \
       reinterpret_cast<hip::Stream*>(stream)->GetCaptureStatus() ==                                \
           hipStreamCaptureStatusActive) {                                                          \
     hipError_t status = hip::capture##name(stream, ##__VA_ARGS__);                                 \
     return status;                                                                                 \
+  } else if (stream != nullptr && stream != hipStreamLegacy &&                                     \
+             reinterpret_cast<hip::Stream*>(stream)->GetCaptureStatus() ==                         \
+                 hipStreamCaptureStatusInvalidated) {                                              \
+    return hipErrorStreamCaptureInvalidated;                                                       \
   }
 
 #define PER_THREAD_DEFAULT_STREAM(stream)                                                         \
-  if (stream == nullptr) {                                                                        \
+  if (stream == nullptr || stream == hipStreamLegacy) {                                           \
     stream = getPerThreadDefaultStream();                                                         \
   }
 
@@ -241,6 +286,7 @@ public:
 
   class Device;
   class MemoryPool;
+  class Event;
   class Stream : public amd::HostQueue {
   public:
     enum Priority : int { High = -1, Normal = 0, Low = 1 };
@@ -305,17 +351,10 @@ public:
     /// Returns the CU mask for the current stream
     const std::vector<uint32_t> GetCUMask() const { return cuMask_; }
 
-    /// Sync all streams
-    static void SyncAllStreams(int deviceId, bool cpu_wait = true);
-
     /// Check whether any blocking stream running
     static bool StreamCaptureBlocking();
 
-    /// Destroy all streams on a given device
-    static void destroyAllStreams(int deviceId);
-
     static void Destroy(hip::Stream* stream);
-
     /// Check Stream Capture status to make sure it is done
     static bool StreamCaptureOngoing(hipStream_t hStream);
 
@@ -415,7 +454,6 @@ public:
         parallelCaptureStreams_.erase(it);
       }
     }
-    static bool existsActiveStreamForDevice(hip::Device* device);
 
     /// The stream should be destroyed via release() rather than delete
     private:
@@ -423,8 +461,11 @@ public:
   };
 
   /// HIP Device class
-  class Device {
+  class Device : public amd::ReferenceCountedObject {
     amd::Monitor lock_{"Device lock", true};
+    // Guards device stream set
+    std::shared_mutex streamSetLock;
+    std::unordered_set<hip::Stream*> streamSet;
     /// ROCclr context
     amd::Context* context_;
     /// Device's ID
@@ -487,7 +528,7 @@ public:
     void setFlags(unsigned int flags) { flags_ = flags; }
     void Reset();
 
-    hip::Stream* NullStream();
+    hip::Stream* NullStream(bool wait = true);
     Stream* GetNullStream() const {return null_stream_;};
 
     void SetActiveStatus() {
@@ -498,7 +539,7 @@ public:
       amd::ScopedLock lock(lock_);
       /// Either stream is active or device is active
       if (isActive_) return true;
-      if (Stream::existsActiveStreamForDevice(this)) {
+      if (existsActiveStreamForDevice()) {
         isActive_ = true;
         return true;
       }
@@ -526,7 +567,7 @@ public:
     void RemoveMemoryPool(MemoryPool* pool);
 
     /// Free memory from the device
-    bool FreeMemory(amd::Memory* memory, Stream* stream);
+    bool FreeMemory(amd::Memory* memory, Stream* stream, Event* event = nullptr);
 
     /// Release freed memory from all pools on the current device
     void ReleaseFreedMemory();
@@ -534,6 +575,27 @@ public:
     /// Removes a destroyed stream from the safe list of memory pools
     void RemoveStreamFromPools(Stream* stream);
 
+    /// Add safe streams into the memppools for reuse
+    void AddSafeStream(Stream* event_stream, Stream* wait_stream);
+
+    /// Returns true if memory pool is valid on this device
+    bool IsMemoryPoolValid(MemoryPool* pool);
+    void AddStream(Stream* stream);
+
+    void RemoveStream(Stream* stream);
+
+    bool StreamExists(Stream* stream);
+
+    void destroyAllStreams();
+
+    void SyncAllStreams( bool cpu_wait = true, bool wait_blocking_streams_only = false);
+
+    bool StreamCaptureBlocking();
+
+    bool existsActiveStreamForDevice();
+  /// Wait all active streams on the blocking queue. The method enqueues a wait command and
+  /// doesn't stall the current thread
+    void WaitActiveStreams(hip::Stream* blocking_stream, bool wait_null_stream = false);
   };
 
   /// Thread Local Storage Variables Aggregator Class
@@ -572,7 +634,7 @@ public:
   /// Get default stream associated with the ROCclr context
   extern hip::Stream* getNullStream(amd::Context&);
   /// Get default stream of the thread
-  extern hip::Stream* getNullStream();
+  extern hip::Stream* getNullStream(bool wait = true);
   /// Get device ID associated with the ROCclr context
   int getDeviceID(amd::Context& ctx);
   /// Check if stream is valid
@@ -583,15 +645,12 @@ public:
 
   extern void WaitThenDecrementSignal(hipStream_t stream, hipError_t status, void* user_data);
 
-  /// Wait all active streams on the blocking queue. The method enqueues a wait command and
-  /// doesn't stall the current thread
-  extern void iHipWaitActiveStreams(hip::Stream* blocking_stream, bool wait_null_stream = false);
-
   extern std::vector<hip::Device*> g_devices;
   extern hipError_t ihipDeviceGetCount(int* count);
   extern int ihipGetDevice();
 
   extern hipError_t ihipMalloc(void** ptr, size_t sizeBytes, unsigned int flags);
+  extern hipError_t ihipHostMalloc(void** ptr, size_t sizeBytes, unsigned int flags);
   extern amd::Memory* getMemoryObject(const void* ptr, size_t& offset, size_t size = 0);
   extern amd::Memory* getMemoryObjectWithOffset(const void* ptr, const size_t size = 0);
   extern void getStreamPerThread(hipStream_t& stream);
