@@ -26,6 +26,8 @@
 #include "platform/runtime.hpp"
 
 #include <unordered_map>
+#include <mutex>
+
 namespace hip {
 constexpr unsigned __hipFatMAGIC2 = 0x48495046;  // "HIPF"
 
@@ -86,8 +88,13 @@ void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void* hostFunctio
     return var ? atoi(var) : 1;
   }()};
   hipError_t hip_error = hipSuccess;
-  hip::Function* func = new hip::Function(std::string(deviceName), modules);
-  hip_error = PlatformState::instance().registerStatFunction(hostFunction, func);
+  // Compiler might share same hostFunction and hence it's needless to have another
+  // hip::Function and hip::Function is stored in map with hostFunction as key.
+  // Creating hip::Function in such case, Leaks it.
+  if (PlatformState::instance().getStatFuncName(hostFunction) == nullptr) {
+    hip::Function* func = new hip::Function(std::string(deviceName), modules);
+    hip_error = PlatformState::instance().registerStatFunction(hostFunction, func);
+  }
   guarantee((hip_error == hipSuccess), "Cannot register Static function, error: %d", hip_error);
 
   if (!enable_deferred_loading) {
@@ -175,6 +182,14 @@ void __hipRegisterTexture(
 }
 
 void __hipUnregisterFatBinary(hip::FatBinaryInfo** modules) {
+  static std::once_flag unregister_device_sync;
+  std::call_once(unregister_device_sync, [](){
+    for (auto& hipDevice : g_devices) {
+      // By synchronizing devices ensure that all HSA signal handlers
+      // complete before removeFatBinary
+      hipDevice->SyncAllStreams(true);
+    }
+  });
   hipError_t err = PlatformState::instance().removeFatBinary(modules);
   guarantee((err == hipSuccess), "Cannot Unregister Fat Binary, error:%d", err);
 }
@@ -414,8 +429,14 @@ hipError_t ihipOccupancyMaxActiveBlocksPerMultiprocessor(
   // If the best block size is smaller than the block size used to fit the maximum,
   // then we need to make the grid bigger for full occupancy.
   const int bestBlocksPerCU = alu_limited_threads / (*bestBlockSize);
+  uint32_t maxCUs = device.info().maxComputeUnits_;
+  if (wrkGrpInfo->isWGPMode_ == false && device.settings().enableWgpMode_ == true) {
+    maxCUs *= 2;
+  } else if ((wrkGrpInfo->isWGPMode_ == true && device.settings().enableWgpMode_ == false)) {
+    maxCUs /= 2;
+  }
   // Unless those blocks are further constrained by LDS size.
-  *numBlocksPerGrid = device.info().maxComputeUnits_ * std::min(bestBlocksPerCU, lds_occupancy_wgs);
+  *numBlocksPerGrid = (maxCUs * std::min(bestBlocksPerCU, lds_occupancy_wgs));
 
   return hipSuccess;
 }
@@ -699,11 +720,8 @@ void PlatformState::init() {
   initialized_ = true;
   for (auto& it : statCO_.modules_) {
     hipError_t err = digestFatBinary(it.first, it.second);
-    if (err == hipErrorNoBinaryForGpu) {
+    if (err != hipSuccess) {
       HIP_ERROR_PRINT(err, "continue parsing remaining modules");
-    } else if (err != hipSuccess) {
-      HIP_ERROR_PRINT(err);
-      return;
     }
   }
   for (auto& it : statCO_.vars_) {
@@ -782,7 +800,7 @@ hipError_t PlatformState::getDynGlobalVar(const char* hostVar, hipModule_t hmod,
                                           hipDeviceptr_t* dev_ptr, size_t* size_ptr) {
   amd::ScopedLock lock(lock_);
 
-  if (hostVar == nullptr || dev_ptr == nullptr || size_ptr == nullptr) {
+  if (hostVar == nullptr) {
     return hipErrorInvalidValue;
   }
 
@@ -791,14 +809,20 @@ hipError_t PlatformState::getDynGlobalVar(const char* hostVar, hipModule_t hmod,
     LogPrintfError("Cannot find the module: 0x%x", hmod);
     return hipErrorNotFound;
   }
-  *dev_ptr = nullptr;
+  if (dev_ptr) {
+    *dev_ptr = nullptr;
+  }
   IHIP_RETURN_ONFAIL(it->second->getManagedVarPointer(hostVar, dev_ptr, size_ptr));
   // if dev_ptr is nullptr, hostvar is not in managed variable list
-  if (*dev_ptr == nullptr) {
+  if ((dev_ptr && *dev_ptr == nullptr) || (size_ptr && *size_ptr == 0)) {
     hip::DeviceVar* dvar = nullptr;
     IHIP_RETURN_ONFAIL(it->second->getDeviceVar(&dvar, hostVar));
-    *dev_ptr = dvar->device_ptr();
-    *size_ptr = dvar->size();
+    if (dev_ptr != nullptr) {
+      *dev_ptr = dvar->device_ptr();
+    }
+    if (size_ptr != nullptr) {
+      *size_ptr = dvar->size();
+    }
   }
   return hipSuccess;
 }
@@ -958,4 +982,27 @@ bool PlatformState::CloseUniqueFileHandle(const std::shared_ptr<UniqueFD>& ufd)
   }
   return true;
 }
+
+void* PlatformState::getDynamicLibraryHandle() {
+  amd::ScopedLock lock(lock_);
+
+  if (dynamicLibraryHandle_ != nullptr) {
+    return dynamicLibraryHandle_;
+  }
+
+#ifdef _WIN32
+  const char* libName = "amdhip64.dll";
+#else
+  const char* libName = "libamdhip64.so";
+#endif
+
+  dynamicLibraryHandle_ = amd::Os::loadLibrary(libName);
+  return dynamicLibraryHandle_;
+}
+
+void PlatformState::setDynamicLibraryHandle(void* handle){
+  amd::ScopedLock lock(lock_);
+  dynamicLibraryHandle_ = handle;
+}
+
 } //namespace hip
