@@ -23,10 +23,9 @@
 #include "hip_event.hpp"
 #include "thread/monitor.hpp"
 #include "hip_prof_api.h"
+#include <atomic>
 
 namespace hip {
-static amd::Monitor streamSetLock{"Guards global stream set"};
-static std::unordered_set<hip::Stream*> streamSet;
 
 // ================================================================================================
 Stream::Stream(hip::Device* dev, Priority p, unsigned int f, bool null_stream,
@@ -43,8 +42,7 @@ Stream::Stream(hip::Device* dev, Priority p, unsigned int f, bool null_stream,
       originStream_(false),
       captureID_(0)
       {
-        amd::ScopedLock lock(streamSetLock);
-        streamSet.insert(this);
+        device_->AddStream(this);
       }
 
 // ================================================================================================
@@ -76,17 +74,14 @@ bool Stream::Create() {
 
 // ================================================================================================
 void Stream::Destroy(hip::Stream* stream) {
-  {
-    amd::ScopedLock lock(streamSetLock);
-    streamSet.erase(stream);
-  }
+  stream->device_->RemoveStream(stream);
   stream->release();
 }
 
 // ================================================================================================
 bool isValid(hipStream_t& stream) {
   // NULL stream is always valid
-  if (stream == nullptr) {
+  if (stream == nullptr || stream == hipStreamLegacy) {
     return true;
   }
 
@@ -95,11 +90,12 @@ bool isValid(hipStream_t& stream) {
   }
 
   hip::Stream* s = reinterpret_cast<hip::Stream*>(stream);
-  amd::ScopedLock lock(streamSetLock);
-  if (streamSet.find(s) == streamSet.end()) {
-    return false;
+  for (auto& device : g_devices) {
+    if (device->StreamExists(s)) {
+      return true;
+    }
   }
-  return true;
+  return false;
 }
 
 // ================================================================================================
@@ -115,162 +111,61 @@ int Stream::DeviceId(const hipStream_t hStream) {
     //return invalid device id
     return -1;
   }
+  bool isNullOrLegacyStream = (hStream == nullptr || hStream == hipStreamLegacy);
   hip::Stream* s = reinterpret_cast<hip::Stream*>(inputStream);
-  int deviceId = (s != nullptr)? s->DeviceId() : ihipGetDevice();
+  int deviceId = isNullOrLegacyStream ? ihipGetDevice() : s->DeviceId();
   assert(deviceId >= 0 && deviceId < static_cast<int>(g_devices.size()));
   return deviceId;
 }
 
 // ================================================================================================
-void Stream::SyncAllStreams(int deviceId, bool cpu_wait) {
-  // Make a local copy to avoid stalls for GPU finish with multiple threads
-  std::vector<hip::Stream*> streams;
-  streams.reserve(streamSet.size());
-  {
-    amd::ScopedLock lock(streamSetLock);
-    for (auto it : streamSet) {
-      if (it->DeviceId() == deviceId) {
-        streams.push_back(it);
-        it->retain();
-      }
-    }
-  }
-  for (auto it : streams) {
-    it->finish(cpu_wait);
-    it->release();
-  }
-  // Release freed memory for all memory pools on the device
-  g_devices[deviceId]->ReleaseFreedMemory();
-}
 
 // ================================================================================================
 bool Stream::StreamCaptureBlocking() {
-  amd::ScopedLock lock(streamSetLock);
-  for (auto& it : streamSet) {
-    if (it->GetCaptureStatus() == hipStreamCaptureStatusActive && it->Flags() != hipStreamNonBlocking) {
+  for (auto& device : g_devices) {
+    if (device->StreamCaptureBlocking()) {
       return true;
     }
   }
   return false;
 }
 
-void Stream::destroyAllStreams(int deviceId) {
-  std::vector<Stream*> toBeDeleted;
-  {
-    amd::ScopedLock lock(streamSetLock);
-    for (auto& it : streamSet) {
-      if (it->Null() == false && it->DeviceId() == deviceId) {
-        toBeDeleted.push_back(it);
-      }
-    }
-  }
-  for (auto& it : toBeDeleted) {
-    hip::Stream::Destroy(it);
-  }
-}
-
 bool Stream::StreamCaptureOngoing(hipStream_t hStream) {
-  hip::Stream* s = reinterpret_cast<hip::Stream*>(hStream);
-  // Allow capture to be less restrictive one one changes the stream capture interaction
-  // mode for the thread
-  if (hip::tls.stream_capture_mode_ == hipStreamCaptureModeRelaxed) {
+  if (hStream == nullptr || hStream == hipStreamLegacy) {
     return false;
   }
-  // If any local thread has an ongoing or concurrent capture sequence initiated
-  // with hipStreamCaptureModeGlobal, it is prohibited from unsafe calls
-  if (s != nullptr && s->GetCaptureMode() == hipStreamCaptureModeGlobal) {
+
+  hip::Stream* s = reinterpret_cast<hip::Stream*>(hStream);
+  if (s->GetCaptureStatus() == hipStreamCaptureStatusNone) {
+    // If current thread is capturing in relaxed mode
+    if (hip::tls.stream_capture_mode_ == hipStreamCaptureModeRelaxed) {
+      return false;
+    }
+    // If any stream in current/concurrent thread is capturing in global mode
     amd::ScopedLock lock(g_captureStreamsLock);
-    return (g_captureStreams.empty() == true && hip::tls.capture_streams_.empty()) ? false : true;
-  }
-  else {
-    amd::ScopedLock lock(g_streamSetLock);
-    return (g_allCapturingStreams.find(s) == g_allCapturingStreams.end() ? false : true);
-  }
-}
-
-bool Stream::existsActiveStreamForDevice(hip::Device* device) {
-
-  amd::ScopedLock lock(streamSetLock);
-
-  for (const auto& active_stream : streamSet) {
-    if ((active_stream->GetDevice() == device) &&
-      active_stream->GetQueueStatus()) {
+    if (!g_captureStreams.empty()) {
+      for (auto stream : hip::g_captureStreams) {
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);
+      }
       return true;
     }
+    // If any stream in current thread is capturing in ThreadLocal mode
+    if (!hip::tls.capture_streams_.empty()) {
+      for (auto stream : hip::tls.capture_streams_) {
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);
+      }
+      return true;
+    }
+    return false;
+  } else if (s->GetCaptureStatus() == hipStreamCaptureStatusActive) {
+    s->SetCaptureStatus(hipStreamCaptureStatusInvalidated);
+    return true;
+  } else if (s->GetCaptureStatus() == hipStreamCaptureStatusInvalidated) {
+    return true;
   }
   return false;
 }
 
-// ================================================================================================
-void iHipWaitActiveStreams(hip::Stream* blocking_stream, bool wait_null_stream) {
-  amd::Command::EventWaitList eventWaitList(0);
-  bool submitMarker = 0;
-
-  auto waitForStream = [&submitMarker,
-                         &eventWaitList](hip::Stream* stream) {
-    if (amd::Command *command = stream->getLastQueuedCommand(true)) {
-      amd::Event &event = command->event();
-      // Check HW status of the ROCcrl event.
-      // Note: not all ROCclr modes support HW status
-      bool ready = stream->device().IsHwEventReady(event);
-      if (!ready) {
-        ready = (command->status() == CL_COMPLETE);
-      }
-      submitMarker |= stream->vdev()->isFenceDirty();
-      // Check the current active status
-      if (!ready) {
-        command->notifyCmdQueue();
-        eventWaitList.push_back(command);
-      } else {
-        command->release();
-      }
-    }
-  };
-
-  if (wait_null_stream) {
-    if (hip::Stream* null_stream = blocking_stream->GetDevice()->GetNullStream()) {
-      waitForStream(null_stream);
-    }
-  } else {
-    amd::ScopedLock lock(streamSetLock);
-
-    for (const auto& active_stream : streamSet) {
-      // If it's the current device
-      if ((&active_stream->device() == &blocking_stream->device()) &&
-        // Make sure it's a default stream
-        ((active_stream->Flags() & hipStreamNonBlocking) == 0) &&
-        // and it's not the current stream
-        (active_stream != blocking_stream)) {
-        // Get the last valid command
-        waitForStream(active_stream);
-      }
-    }
-  }
-
-  // Check if we have to wait anything
-  if (eventWaitList.size() > 0 || submitMarker) {
-    amd::Command* command = new amd::Marker(*blocking_stream, kMarkerDisableFlush, eventWaitList);
-    if (command != nullptr) {
-      command->enqueue();
-      command->release();
-    }
-
-    //Reset the dirty flag for all streams now that the marker is submitted
-    amd::ScopedLock lock(streamSetLock);
-    for (const auto& stream : streamSet) {
-      amd::HostQueue* active_queue = stream->asHostQueue();
-      if (active_queue->vdev()->isFenceDirty()) {
-        active_queue->vdev()->resetFenceDirty();
-      }
-    }
-  }
-
-  // Release all active commands. It's safe after the marker was enqueued
-  for (const auto& it : eventWaitList) {
-    it->release();
-  }
-}
-
 // ================================================================================================
 void CL_CALLBACK ihipStreamCallback(cl_event event, cl_int command_exec_status, void* user_data) {
   StreamCallback* cbo = reinterpret_cast<StreamCallback*>(user_data);
@@ -444,22 +339,30 @@ hipError_t hipStreamSynchronize_common(hipStream_t stream) {
   if (!hip::isValid(stream)) {
     HIP_RETURN(hipErrorContextIsDestroyed);
   }
-  if (stream != nullptr) {
+  if (stream != nullptr && stream != hipStreamLegacy) {
     // If still capturing return error
     if (hip::Stream::StreamCaptureOngoing(stream) == true) {
       HIP_RETURN(hipErrorStreamCaptureUnsupported);
     }
   }
-  bool wait = (stream == nullptr) ? true : false;
-  constexpr bool kDontWaitForCpu = false;
 
-  auto hip_stream = hip::getStream(stream, wait);
+  if (stream == nullptr) {
+    // Do cpu wait on null stream and only on blocking streams
+    constexpr bool WaitblockingStreamOnly = true;
+    getCurrentDevice()->SyncAllStreams(true, WaitblockingStreamOnly);
 
-  // Wait for the current host queue
-  hip_stream->finish(kDontWaitForCpu);
+    // Release freed memory for all memory pools on the device
+    getCurrentDevice()->ReleaseFreedMemory();
+  } else {
+    constexpr bool wait = false;
+    auto hip_stream = hip::getStream(stream, wait);
 
-  // Release freed memory for all memory pools on the device
-  hip_stream->GetDevice()->ReleaseFreedMemory();
+    // Wait for the current host queue
+    hip_stream->finish();
+
+    // Release freed memory for all memory pools on the device
+    hip_stream->GetDevice()->ReleaseFreedMemory();
+  }
   return hipSuccess;
 }
 
@@ -505,6 +408,13 @@ hipError_t hipStreamDestroy(hipStream_t stream) {
       g_captureStreams.erase(g_it);
     }
   }
+  {
+    amd::ScopedLock lock(g_streamSetLock);
+    const auto& g_it = std::find(g_allCapturingStreams.begin(), g_allCapturingStreams.end(), s);
+    if (g_it != g_allCapturingStreams.end()) {
+      g_allCapturingStreams.erase(g_it);
+    }
+  }
   const auto& l_it = std::find(hip::tls.capture_streams_.begin(),
                       hip::tls.capture_streams_.end(), s);
   if (l_it != hip::tls.capture_streams_.end()) {
@@ -534,8 +444,15 @@ hipError_t hipStreamWaitEvent_common(hipStream_t stream, hipEvent_t event, unsig
   }
   hip::Stream* waitStream = reinterpret_cast<hip::Stream*>(stream);
   hip::Event* e = reinterpret_cast<hip::Event*>(event);
-  hip::Stream* eventStream = reinterpret_cast<hip::Stream*>(e->GetCaptureStream());
+  auto eventStreamHandle = reinterpret_cast<hipStream_t>(e->GetCaptureStream());
+  // the stream associated with the device might have been destroyed
+  if (!hip::isValid(eventStreamHandle)) {
+    // Stream associated with the event has been released
+    // meaning the event has been completed and we can resume the current stream
+    return hipSuccess;
+  }
 
+  hip::Stream* eventStream = reinterpret_cast<hip::Stream*>(eventStreamHandle);
   if (eventStream != nullptr && eventStream->IsEventCaptured(event) == true) {
     ClPrint(amd::LOG_INFO, amd::LOG_API,
           "[hipGraph] Current capture node StreamWaitEvent on stream : %p, Event %p", stream,
@@ -555,10 +472,15 @@ hipError_t hipStreamWaitEvent_common(hipStream_t stream, hipEvent_t event, unsig
     if (flags != 0) {
       return hipErrorInvalidValue;
     }
-    if ((eventStream != nullptr) &&
-      (eventStream->GetCaptureStatus() == hipStreamCaptureStatusActive)) {
-      // If stream is capturing but event is not recorded on event's stream.
-      return hipErrorStreamCaptureIsolation;
+    if (eventStream != nullptr) {
+      if (eventStream->GetCaptureStatus() == hipStreamCaptureStatusActive) {
+        // If stream is capturing but event is not recorded on event's stream.
+        return hipErrorStreamCaptureIsolation;
+      }
+      if ((waitStream != nullptr && stream != hipStreamLegacy) &&
+          (eventStream->DeviceId() == waitStream->DeviceId())) {
+        eventStream->GetDevice()->AddSafeStream(eventStream, waitStream);
+      }
     }
     status = e->streamWait(stream, flags);
   }
@@ -592,6 +514,14 @@ hipError_t hipStreamQuery_common(hipStream_t stream) {
   bool wait = (stream == nullptr) ? true : false;
   hip::Stream* hip_stream = hip::getStream(stream, wait);
 
+  if (hip_stream->vdev()->isFenceDirty()) {
+    amd::Command* command = new amd::Marker(*hip_stream, kMarkerDisableFlush);
+    if (command != nullptr) {
+      command->enqueue();
+      command->release();
+    }
+  }
+
   amd::Command* command = hip_stream->getLastQueuedCommand(true);
   if (command == nullptr) {
     // Nothing was submitted to the queue
@@ -602,6 +532,7 @@ hipError_t hipStreamQuery_common(hipStream_t stream) {
   if (command->type() != 0) {
     event.notifyCmdQueue();
   }
+
   // Check HW status of the ROCcrl event. Note: not all ROCclr modes support HW status
   bool ready = command->queue()->device().IsHwEventReady(event);
   if (!ready) {
@@ -661,13 +592,13 @@ hipError_t streamCallback_common(hipStream_t stream, StreamCallback* cbo, void*
     return hipErrorInvalidValue;
   }
   block_command->enqueue();
-  block_command->release();
 
   // Release the callback marker
   command->release();
   // Notify the command queue about a possible waiter for the calback
   block_command->notifyCmdQueue();
 
+  block_command->release();
   return hipSuccess;
 }
 
