@@ -33,13 +33,21 @@ THE SOFTWARE.
 
 namespace hiprtc {
 using namespace helpers;
+std::unordered_set<RTCLinkProgram*>RTCLinkProgram::linker_set_;
 
 std::vector<std::string> getLinkOptions(const LinkArguments& args) {
   std::vector<std::string> res;
-  auto irArgCount = args.linkerIRArgCount();
+
+  {  // process optimization level
+    std::string opt("-O");
+    opt += std::to_string(args.optimization_level_);
+    res.push_back(opt);
+  }
+
+  const auto irArgCount = args.linker_ir2isa_args_count_;
   if (irArgCount > 0) {
     res.reserve(irArgCount);
-    auto irArg = args.linkerIRArg();
+    const auto irArg = args.linker_ir2isa_args_;
     for (size_t i = 0; i < irArgCount; i++) {
       res.emplace_back(std::string(irArg[i]));
     }
@@ -57,9 +65,12 @@ RTCProgram::RTCProgram(std::string name) : name_(name) {
 }
 
 bool RTCProgram::findIsa() {
+
+#ifdef BUILD_SHARED_LIBS
   const char* libName;
 #ifdef _WIN32
-  libName = "amdhip64.dll";
+  std::string dll_name = std::string("amdhip64_" + std::to_string(HIP_VERSION_MAJOR) + ".dll");
+  libName = dll_name.c_str();
 #else
   libName = "libamdhip64.so";
 #endif
@@ -112,6 +123,23 @@ bool RTCProgram::findIsa() {
 
   amd::Os::unloadLibrary(handle);
   return true;
+
+#else
+  int device;
+  hipError_t status = hipGetDevice(&device);
+  if (status != hipSuccess) {
+    return false;
+  }
+  hipDeviceProp_t props;
+  status = hipGetDeviceProperties(&props, device);
+  if (status != hipSuccess) {
+    return false;
+  }
+  isa_ = "amdgcn-amd-amdhsa--";
+  isa_.append(props.gcnArchName);
+
+  return true;
+#endif
 }
 
 // RTC Compile Program Member Functions
@@ -156,11 +184,7 @@ RTCCompileProgram::RTCCompileProgram(std::string name_) : RTCProgram(name_), fgp
   compile_options_.push_back(hipVerMajor);
   compile_options_.push_back(hipVerMinor);
   compile_options_.push_back(hipVerPatch);
-  compile_options_.push_back("-D__HIPCC_RTC__");
-  compile_options_.push_back("-include");
-  compile_options_.push_back("hiprtc_runtime.h");
   compile_options_.push_back("-std=c++14");
-  compile_options_.push_back("-nogpuinc");
   compile_options_.push_back("-Wno-gnu-line-marker");
   compile_options_.push_back("-Wno-missing-prototypes");
 #ifdef _WIN32
@@ -170,8 +194,6 @@ RTCCompileProgram::RTCCompileProgram(std::string name_) : RTCProgram(name_), fgp
   compile_options_.push_back("-fms-compatibility");
 #endif
   AppendCompileOptions();
-
-  exe_options_.push_back("-O3");
 }
 
 bool RTCCompileProgram::addSource(const std::string& source, const std::string& name) {
@@ -394,6 +416,16 @@ RTCLinkProgram::RTCLinkProgram(std::string name) : RTCProgram(name) {
   if (amd::Comgr::create_data_set(&link_input_) != AMD_COMGR_STATUS_SUCCESS) {
     crashWithMessage("Failed to allocate internal hiprtc structure");
   }
+  amd::ScopedLock lock(lock_);
+  linker_set_.insert(this);
+}
+
+bool RTCLinkProgram::isLinkerValid(RTCLinkProgram* link_program) {
+  amd::ScopedLock lock(lock_);
+  if (linker_set_.find(link_program) == linker_set_.end()) {
+    return false;
+  }
+  return true;
 }
 
 bool RTCLinkProgram::AddLinkerOptions(unsigned int num_options, hiprtcJIT_option* options_ptr,
@@ -626,7 +658,6 @@ bool RTCLinkProgram::LinkComplete(void** bin_out, size_t* size_out) {
   }
 
   std::vector<std::string> exe_options = getLinkOptions(link_args_);
-  exe_options.push_back("-O3");
   LogPrintfInfo("Exe options forwarded to compiler: %s",
                 [&]() {
                   std::string ret;
@@ -638,7 +669,7 @@ bool RTCLinkProgram::LinkComplete(void** bin_out, size_t* size_out) {
                 }()
                     .c_str());
   if (!createExecutable(exec_input_, isa_, exe_options, build_log_, executable_)) {
-    LogError("Error in hiprtc: unable to create exectuable");
+    LogPrintfInfo("Error in hiprtc: unable to create exectuable: %s", build_log_.c_str());
     return false;
   }
 
