@@ -57,6 +57,7 @@
 #include <set>
 #include <unordered_set>
 #include <utility>
+#include <shared_mutex>
 
 namespace amd {
 class Command;
@@ -244,7 +245,7 @@ inline void WriteAqlArgAt(unsigned char* dst, //!< The write pointer to the buff
   *(reinterpret_cast<T*>(dst + offset)) = src;
 }
 
-namespace device {
+namespace amd::device {
 class ClBinary;
 class BlitManager;
 class Program;
@@ -412,10 +413,13 @@ struct Info : public amd::EmbeddedObject {
   //  the device implement error correction.
   uint32_t errorCorrectionSupport_;
 
-  //! true if the device and the host have a unified memory subsystem and
-  //  is false otherwise.
+  //! true if the device and the host have a unified memory and is false otherwise.
   uint32_t hostUnifiedMemory_;
 
+  //! true if the device and the host have a unified memory management subsystem and
+  //  is false otherwise.
+  uint32_t iommuv2_;
+
   //! Describes the resolution of device timer.
   size_t profilingTimerResolution_;
 
@@ -648,6 +652,17 @@ struct Info : public amd::EmbeddedObject {
 //! Device settings
 class Settings : public amd::HeapObject {
  public:
+
+  enum KernelArgImpl {
+    HostKernelArgs = 0,       //!< Kernel Arguments are put into host memory
+    DeviceKernelArgs,         //!< Device memory kernel arguments with no memory
+                              //!< ordering workaround (e.g. XGMI)
+    DeviceKernelArgsReadback, //!< Device memory kernel arguments with kernel
+                              //!< argument readback workaround
+    DeviceKernelArgsHDP       //!< Device memory kernel arguments with kernel
+                              //!< argument readback plus HDP flush workaround.
+  };
+
   uint64_t extensions_;  //!< Supported OCL extensions
   union {
     struct {
@@ -671,7 +686,9 @@ class Settings : public amd::HeapObject {
       uint fenceScopeAgent_ : 1;      //!< Enable fence scope agent in AQL dispatch packet
       uint rocr_backend_ : 1;         //!< Device uses ROCr backend for submissions
       uint gwsInitSupported_:1;       //!< Check if GWS is supported on this machine.
-      uint reserved_ : 10;
+      uint kernel_arg_opt_: 1;        //!< Enables kernel arg optimization for blit kernels
+      uint kernel_arg_impl_ : 2;      //!< Kernel argument implementation
+      uint reserved_ : 7;
     };
     uint value_;
   };
@@ -710,6 +727,13 @@ class Memory : public amd::HeapObject {
     CpuWriteOnly = 0x00000002,  //!< Lock for CPU write only operation
   };
 
+  //! Memory Access flags at device level
+  enum MemAccess {
+    kMemAccessNone = 0,        //! No Access
+    kMemAccessRead = 1,        //! Read Access
+    kMemAccessReadWrite = 3    //! Read and Write Access
+  };
+
   union SyncFlags {
     struct {
       uint skipParent_ : 1;  //!< Skip parent synchronization
@@ -750,11 +774,14 @@ class Memory : public amd::HeapObject {
   Memory(amd::Memory& owner)
       : flags_(0), owner_(&owner), version_(0), mapMemory_(NULL), indirectMapCount_(0) {
     size_ = owner.getSize();
+    memAccess_ = MemAccess::kMemAccessNone;
   }
 
   //! Constructor (no owner), always eager allocation.
   Memory(size_t size)
-      : flags_(0), owner_(NULL), version_(0), mapMemory_(NULL), indirectMapCount_(0), size_(size) {}
+      : flags_(0), owner_(NULL), version_(0), mapMemory_(NULL), indirectMapCount_(0), size_(size) {
+    memAccess_ = MemAccess::kMemAccessNone;
+  }
 
   enum GLResourceOP {
     GLDecompressResource = 0,  // orders the GL driver to decompress any depth-stencil or MSAA
@@ -905,6 +932,12 @@ class Memory : public amd::HeapObject {
     }
   }
 
+  //! Set access to the memory in this device.
+  void SetAccess(MemAccess memAccess) { memAccess_ = memAccess; }
+
+  //! Get current access of the memory in device.
+  MemAccess GetAccess() const { return memAccess_; }
+
  protected:
   enum Flags {
     HostMemoryDirectAccess = 0x00000001,  //!< GPU has direct access to the host memory
@@ -918,6 +951,8 @@ class Memory : public amd::HeapObject {
   };
   uint flags_;  //!< Memory object flags
 
+  MemAccess memAccess_; //!< Memory Access flag
+
   amd::Memory* owner_;  //!< The Memory instance that we cache,
                         //!< or NULL if we're device-private workspace.
 
@@ -1287,14 +1322,17 @@ class VirtualDevice : public amd::HeapObject {
   //! Returns true if device has active wait setting
   bool ActiveWait() const;
 
-  //! Returns the status of queue handler callback
-  virtual bool isHandlerPending() const = 0;
-
   //! Returns fence state of the VirtualGPU
   virtual bool isFenceDirty() const = 0;
-
+  //! Init hidden heap for device memory allocations
+  virtual void HiddenHeapInit() = 0;
   //! Dispatch captured AQL packet
-  virtual bool dispatchAqlPacket(uint8_t* aqlpacket, amd::AccumulateCommand* vcmd = nullptr) = 0;
+  virtual bool dispatchAqlPacket(uint8_t* aqlpacket,
+                                 const std::string& kernelName,
+                                 amd::AccumulateCommand* vcmd = nullptr) = 0;
+
+  //! Returns the number of outstanding HSA async handlers
+  std::atomic<uint64_t>& QueuedAsyncHandlers() const { return queued_async_handlers_; }
 
  private:
   //! Disable default copy constructor
@@ -1311,36 +1349,42 @@ class VirtualDevice : public amd::HeapObject {
 
   amd::Monitor execution_;  //!< Lock to serialise access to all device objects
   uint index_;              //!< The virtual device unique index
+  mutable std::atomic<uint64_t> queued_async_handlers_ = 0; //!< Outstanding HSA async handlers
 };
 
-}  // namespace device
+}  // namespace amd::device
 
 namespace amd {
 
 //! MemoryObject map lookup  class
 class MemObjMap : public AllStatic {
  public:
-  static size_t size();  //!< obtain the size of the container
-  static void AddMemObj(const void* k,
-                        amd::Memory* v);  //!< add the host mem pointer and buffer in the container
-  static void RemoveMemObj(const void* k);  //!< Remove an entry of mem object from the container
-  static amd::Memory* FindMemObj(
-      const void* k,              //!< find the mem object based on the input pointer
-      size_t* offset = nullptr);  //!< Offset in the memory location
+  //!< add the host mem pointer and buffer in the container
+  static void AddMemObj(const void* k, amd::Memory* v);
+
+  //!< Remove an entry of mem object from the container
+  static void RemoveMemObj(const void* k);
+
+  //!< Find the mem object based on the input pointer, outputs the offset
+  static amd::Memory* FindMemObj( const void* k, size_t* offset = nullptr);
   static void UpdateAccess(amd::Device *peerDev);
-  static void Purge(amd::Device* dev); //!< Purge all user allocated memories on the given device
+  //!< Purge all user allocated memories on the given device
+  static void Purge(amd::Device* dev);
+  //!< Same as AddMemObj but for virtual addressing
+  static void AddVirtualMemObj(const void* k, amd::Memory* v);
+
+  //!< Same as RemoveMemObj but for virtual addressing
+  static void RemoveVirtualMemObj(const void* k);
+  //!< Same as FindMemObj but for virtual addressing
+  static amd::Memory* FindVirtualMemObj(const void* k);
 
-  static void AddVirtualMemObj(const void* k,
-                               amd::Memory* v);  //!< Same as AddMemObj but for virtual addressing
-  static void RemoveVirtualMemObj(const void* k);  //!< Same as RemoveMemObj but for virtual addressing
-  static amd::Memory* FindVirtualMemObj(
-      const void* k);  //!< Same as FindMemObj but for virtual addressing
  private:
-  static std::map<uintptr_t, amd::Memory*>
-      MemObjMap_;                      //!< the mem object<->hostptr information container
-  static std::map<uintptr_t, amd::Memory*>
-      VirtualMemObjMap_;               //!< the virtual mem object<->hostptr information container
-  static amd::Monitor AllocatedLock_;  //!< amd monitor locker
+  //!< the mem object<->hostptr information container
+  static std::map<uintptr_t, amd::Memory*> MemObjMap_;
+  //!< the virtual mem object<->hostptr information container
+  static std::map<uintptr_t, amd::Memory*> VirtualMemObjMap_;
+  //!< Shared read/write lock
+  static std::shared_mutex AllocatedLock_;
 };
 
 /// @brief Instruction Set Architecture properties.
@@ -1754,7 +1798,7 @@ class Device : public RuntimeObject {
   }
 
   virtual void* deviceLocalAlloc(size_t size, bool atomics = false,
-                                 bool pseudo_fine_grain = false) const {
+                                 bool pseudo_fine_grain = false, bool contiguous = false) const {
     ShouldNotCallThis();
     return NULL;
   }
@@ -1793,6 +1837,34 @@ class Device : public RuntimeObject {
    */
   virtual void svmFree(void* ptr) const = 0;
 
+  /**
+   * Validatates Virtual Address range between parent and sub-buffer.
+   *
+   * @param vaddr_base_obj Parent/base object of the virtual address.
+   * @param vaddr_sub_obj Sub Buffer object of the virtual address.
+   */
+  static bool ValidateVirtualAddressRange(amd::Memory* vaddr_base_obj, amd::Memory* vaddr_sub_obj);
+
+  /**
+   * Abstracts the Virtual Buffer creation and memobj/virtual memobj add/delete logic.
+   *
+   * @param device_context Context the virtual buffer should be created.
+   * @param vptr virtual ptr to store in the buffer object.
+   * @param size Size of the buffer
+   * @param deviceId deviceId
+   * @param parent base_obj or sub_obj
+   * @param ForceAlloc force_alloc
+   */
+  amd::Memory* CreateVirtualBuffer(Context& device_context, void* vptr, size_t size,
+                                           int deviceId, bool parent, bool kForceAlloc = false);
+
+  /**
+   * Deletes Virtual Buffer and creates memob
+   *
+   * @param vaddr_mem_obj amd::Memory object of parent/sub buffer.
+   */
+  bool DestroyVirtualBuffer(amd::Memory* vaddr_mem_obj);
+
   /**
    * Reserve a VA range with no backing store
    *
@@ -1810,8 +1882,7 @@ class Device : public RuntimeObject {
    * @param access_flags Access permissions
    * @param count Number of access permissions
    */
-  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags,
-                            size_t count) = 0;
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags) = 0;
 
   /**
    * Get Access permisions for a virtual memory object.
@@ -1826,7 +1897,31 @@ class Device : public RuntimeObject {
    *
    * @param addr Start address of the range
    */
-  virtual void virtualFree(void* addr) = 0;
+  virtual bool virtualFree(void* addr) = 0;
+
+  /**
+   * Export Shareable VMM Handle to FD
+   *
+   * @param amd_mem_obj amd::Memory obj which holds the hsa_handle.
+   * @param flags any flags to be passed
+   * @param shareableHandle exported handle, points to fdesc.
+   */
+  virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags,
+                                        void* shareableHandle) {
+    ShouldNotCallThis();
+    return false;
+  }
+
+  /**
+   * Import FD from Shareable VMM Handle
+   *
+   * @param osHandle os handle/fdesc/void*
+   * @param amd_mem_obj amd_mem_obj with hsa_handle/memory_obj.
+   */
+  virtual amd::Memory* ImportShareableVMMHandle(void* osHandle) {
+    ShouldNotCallThis();
+    return nullptr;
+  }
 
   /**
    * @return True if the device successfully applied the SVM attributes in HMM for device memory
@@ -1861,7 +1956,9 @@ class Device : public RuntimeObject {
   // Returns the status of HW event, associated with amd::Event
   virtual bool IsHwEventReady(
       const amd::Event& event,    //!< AMD event for HW status validation
-      bool wait = false) const {  //!< If true then forces the event completion
+      bool wait = false,          //!< If true then forces the event completion
+      uint32_t hip_event_flags = 0 //!< flags associated with the event. 0 = hipEventDefault
+      ) const {
     return false;
   };
 
@@ -1992,6 +2089,24 @@ class Device : public RuntimeObject {
     return false;
   }
 
+  //! Returns the queues that have at least one submitted command
+  std::vector<amd::CommandQueue*> getActiveQueues() {
+     amd::ScopedLock lock(activeQueuesLock_);
+     return std::vector<amd::CommandQueue*>(activeQueues.begin(), activeQueues.end());
+  }
+
+  //! Adds the queue to the set of active command queues
+  void addToActiveQueues(amd::CommandQueue* commandQueue) {
+     amd::ScopedLock lock(activeQueuesLock_);
+     activeQueues.insert(commandQueue);
+  }
+
+  //! Removes the queue from the set of active command queues
+  void removeFromActiveQueues(amd::CommandQueue* commandQueue) {
+    amd::ScopedLock lock(activeQueuesLock_);
+    activeQueues.erase(commandQueue);
+  }
+
   // Notifies device about context destroy
   virtual void ContextDestroy() {}
 
@@ -2032,13 +2147,17 @@ class Device : public RuntimeObject {
   static Memory* p2p_stage_;          //!< Staging resources
   std::vector<Device*> enabled_p2p_devices_;  //!< List of user enabled P2P devices for this device
 
-  std::once_flag heap_initialized_; //!< Heap buffer initialization flag
+  std::once_flag heap_initialized_;  //!< Heap buffer initialization flag
+  std::once_flag heap_allocated_;    //!< Heap buffer allocation flag
+
   device::Memory* heap_buffer_;     //!< Preallocated heap buffer for memory allocations on device
 
   amd::Memory* arena_mem_obj_;      //!< Arena memory object
   uint64_t stack_size_{1024};       //!< Device stack size
   device::Memory* initial_heap_buffer_;   //!< Initial heap buffer
   uint64_t initial_heap_size_{HIP_INITIAL_DM_SIZE};  //!< Initial device heap size
+  amd::Monitor activeQueuesLock_ {"Guards access to the activeQueues set"};
+  std::unordered_set<amd::CommandQueue*> activeQueues; //!< The set of active queues
  private:
   const Isa *isa_;                //!< Device isa
   bool IsTypeMatching(cl_device_type type, bool offlineDevices);
