@@ -38,7 +38,7 @@
 #include "hsailctx.hpp"
 #endif
 
-namespace device {
+namespace amd::device {
 
 // ================================================================================================
 static constexpr clk_value_type_t ClkValueMapType[6][6] = {
@@ -390,6 +390,9 @@ static amd_comgr_status_t populateArgsV3(const amd_comgr_metadata_node_t key,
       {
         auto itValueKind = ArgValueKindV3.find(buf);
         if (itValueKind == ArgValueKindV3.end()) {
+          LogPrintfError("Unknown Kernel arg metadata: %s", buf.c_str());
+          LogError("This may be due to running HIP app that requires a new HIP runtime version");
+          LogError("Please update the display driver");
           return AMD_COMGR_STATUS_ERROR;
         }
         lcArg->info_.oclObject_ = itValueKind->second;
@@ -617,6 +620,7 @@ Kernel::Kernel(const amd::Device& dev, const std::string& name, const Program& p
   workGroupInfo_.compileSizeHint_[1] = 0;
   workGroupInfo_.compileSizeHint_[2] = 0;
   workGroupInfo_.compileVecTypeHint_ = "";
+  workGroupInfo_.isWGPMode_ = false;
   workGroupInfo_.uniformWorkGroupSize_ = false;
   workGroupInfo_.wavesPerSimdHint_ = 0;
   workGroupInfo_.constMemSize_ = 0;
@@ -1477,7 +1481,7 @@ void Kernel::InitPrintf(const std::vector<std::string>& printfInfoStrings) {
     } while (end != std::string::npos);
 
     if (tokens.size() < 2) {
-      ClPrint(amd::LOG_WARNING, amd::LOG_KERN, "Invalid PrintInfo string: \"%s\"", str.c_str());
+      LogPrintfError("Invalid PrintInfo string: \"%s\"", str.c_str());
       continue;
     }
 
@@ -1503,7 +1507,7 @@ void Kernel::InitPrintf(const std::vector<std::string>& printfInfoStrings) {
 
     // ensure that we have the correct number of tokens
     if (tokens.size() < end + 1 /*last token is the fmtString*/) {
-      ClPrint(amd::LOG_WARNING, amd::LOG_KERN, "Invalid PrintInfo string: \"%s\"", str.c_str());
+      LogPrintfError("Invalid PrintInfo string: \"%s\"", str.c_str());
       continue;
     }
 
@@ -1643,4 +1647,4 @@ void Kernel::InitPrintf(const aclPrintfFmt* aclPrintf) {
   }
 }
 #endif // defined(WITH_COMPILER_LIB)
-}
+} // namespace amd::device
