@@ -349,7 +349,7 @@ namespace amd {
   }  // hsa
 }  // amd
 
-namespace device {
+namespace amd::device {
 
 class Program;
 
@@ -500,7 +500,7 @@ class Kernel : public amd::HeapObject {
   const uint32_t KernargSegmentByteSize() const { return kernargSegmentByteSize_; }
   void SetKernargSegmentByteSize(uint32_t size) { kernargSegmentByteSize_ = size; }
 
-  const uint8_t KernargSegmentAlignment() const { return kernargSegmentAlignment_; }
+  const uint32_t KernargSegmentAlignment() const { return kernargSegmentAlignment_; }
   void SetKernargSegmentAlignment(uint32_t align) { kernargSegmentAlignment_ = align; }
 
   void SetSymbolName(const std::string& name) { symbolName_ = name; }
@@ -601,4 +601,4 @@ class Kernel : public amd::HeapObject {
 #if defined(USE_COMGR_LIBRARY)
 amd_comgr_status_t getMetaBuf(const amd_comgr_metadata_node_t meta, std::string* str);
 #endif // defined(USE_COMGR_LIBRARY)
-} // namespace device
+} // namespace amd::device
