@@ -34,7 +34,7 @@
 #include <ctime>
 #include <algorithm>
 
-namespace pal {
+namespace amd::pal {
 
 void HSAILKernel::setWorkGroupInfo(const uint32_t privateSegmentSize,
                                    const uint32_t groupSegmentSize, const uint16_t numSGPRs,
@@ -337,8 +337,7 @@ hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(
         if (amd::IS_HIP) {
           uintptr_t buffer = reinterpret_cast<uintptr_t>(gpu.getOrCreateHostcallBuffer());
           if (!buffer) {
-            ClPrint(amd::LOG_ERROR, amd::LOG_KERN,
-                    "Kernel expects a hostcall buffer, but none found");
+            LogError("Kernel expects a hostcall buffer, but none found");
           }
           assert(it.size_ == sizeof(buffer) && "check the sizes");
           WriteAqlArgAt(hidden_arguments, buffer, it.size_, it.offset_);
@@ -571,4 +570,4 @@ bool LightningKernel::postLoad() {
 }
 #endif  // defined(USE_COMGR_LIBRARY)
 
-}  // namespace pal
+}  // namespace amd::pal
