@@ -55,7 +55,7 @@
  */
 
 //! HSA Device Implementation
-namespace roc {
+namespace amd::roc {
 
 /**
  * @brief List of environment variables that could be used to
@@ -82,6 +82,7 @@ public:
   Timestamp*    ts_;      //!< Timestamp object associated with the signal
   HwQueueEngine engine_;  //!< Engine used with this signal
   amd::Monitor  lock_;    //!< Signal lock for update
+  bool isPacketDispatch_; //!< True if the packet associated with the signal is dispatch
 
   typedef union {
     struct {
@@ -98,6 +99,7 @@ public:
     : ts_(nullptr)
     , engine_(HwQueueEngine::Compute)
     , lock_("Signal Ops Lock", true)
+    , isPacketDispatch_(false)
     {
       signal_.handle = 0;
       flags_.done_ = true;
@@ -221,17 +223,18 @@ class NullDevice : public amd::Device {
     ShouldNotReachHere();
     return;
   }
+
   void* virtualAlloc(void* req_addr, size_t size, size_t alignment) override {
     ShouldNotReachHere();
     return nullptr;
   }
 
-  void virtualFree(void* addr) override {
+  bool virtualFree(void* addr) override {
     ShouldNotReachHere();
-    return;
+    return true;
   }
 
-  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags, size_t count)
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags)
                             override {
     ShouldNotReachHere();
     return false;
@@ -284,7 +287,11 @@ class NullDevice : public amd::Device {
     return true;
   }
 
-  bool IsHwEventReady(const amd::Event& event, bool wait = false) const override { return false; }
+  bool IsHwEventReady(const amd::Event& event, bool wait = false,
+                      uint32_t hip_event_flags = 0) const override {
+    return false;
+  }
+
   bool IsHwEventReadyForcedWait(const amd::Event& event) const override { return false; }
   void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const override{};
   void ReleaseGlobalSignal(void* signal) const override {}
@@ -450,8 +457,10 @@ class Device : public NullDevice {
   bool deviceAllowAccess(void* dst) const;
 
   bool allowPeerAccess(device::Memory* memory) const;
+  void deviceVmemRelease(uint64_t mem_handle) const;
   uint64_t deviceVmemAlloc(size_t size, uint64_t flags) const;
-  void* deviceLocalAlloc(size_t size, bool atomics = false, bool pseudo_fine_grain=false) const;
+  void* deviceLocalAlloc(size_t size, bool atomics = false, bool pseudo_fine_grain=false,
+                         bool contiguous = false) const;
 
   void memFree(void* ptr, size_t size) const;
 
@@ -466,15 +475,22 @@ class Device : public NullDevice {
                                 size_t num_attributes, const void* dev_ptr, size_t count) const;
 
   virtual void* virtualAlloc(void* req_addr, size_t size, size_t alignment);
-  virtual void virtualFree(void* addr);
+  virtual bool virtualFree(void* addr);
 
-  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags, size_t count);
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags);
   virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr);
 
+  virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags, void* shareableHandle);
+
+  bool ImportShareableHSAHandle(void* osHandle, uint64_t* hsa_handle_ptr) const;
+
+  virtual amd::Memory* ImportShareableVMMHandle(void* osHandle);
+
   virtual bool SetClockMode(const cl_set_device_clock_mode_input_amd setClockModeInput,
                             cl_set_device_clock_mode_output_amd* pSetClockModeOutput);
 
-  virtual bool IsHwEventReady(const amd::Event& event, bool wait = false) const;
+  virtual bool IsHwEventReady(const amd::Event& event, bool wait = false,
+                              uint32_t hip_event_flags = 0) const;
   virtual bool IsHwEventReadyForcedWait(const amd::Event& event) const;
   virtual void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const;
   virtual void ReleaseGlobalSignal(void* signal) const;
@@ -580,9 +596,11 @@ class Device : public NullDevice {
 
   //! Allocates hidden heap for device memory allocations
   void HiddenHeapAlloc(const VirtualGPU& gpu);
-
+  //! Init hidden heap for device memory allocations
+  void HiddenHeapInit(const VirtualGPU& gpu);
   uint32_t fetchSDMAMask(const device::BlitManager* handle, bool readEngine = true) const;
-  void resetSDMAMask(const device::BlitManager* handle) const ;
+  void resetSDMAMask(const device::BlitManager* handle) const;
+  void getSdmaRWMasks(uint32_t* readMask, uint32_t* writeMask) const;
   bool isXgmi() const { return isXgmi_; }
 
  private:
@@ -611,7 +629,7 @@ class Device : public NullDevice {
   mutable std::mutex lock_allow_access_; //!< To serialize allow_access calls
   hsa_agent_t bkendDevice_;
   uint32_t pciDeviceId_;
-  hsa_agent_t* p2p_agents_list_;
+  hsa_agent_t* p2p_agents_list_ = nullptr;
   hsa_profile_t agent_profile_;
   hsa_amd_memory_pool_t group_segment_;
   hsa_amd_memory_pool_t system_segment_;
@@ -673,7 +691,7 @@ class Device : public NullDevice {
 #endif
 #endif
 };                                // class roc::Device
-}  // namespace roc
+}  // namespace amd::roc
 
 /**
  * @}
