@@ -1,4 +1,4 @@
-/* Copyright (c) 2010 - 2021 Advanced Micro Devices, Inc.
+/* Copyright (c) 2010 - 2024 Advanced Micro Devices, Inc.
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -18,13 +18,6 @@
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE. */
 
-/*! \file command.hpp
- *  \brief  Declarations for Event, Command and HostQueue objects.
- *
- *  \author Laurent Morichetti
- *  \date   October 2008
- */
-
 #ifndef COMMAND_HPP_
 #define COMMAND_HPP_
 
@@ -62,6 +55,7 @@ namespace amd {
 
 class Command;
 class HostQueue;
+union ComputeCommand;
 
 /*! \brief Encapsulates the status of a command.
  *
@@ -108,7 +102,7 @@ class Event : public RuntimeObject {
       : enabled_(enabled), marker_ts_(false) {
       if (enabled) {
         clear();
-        correlation_id_ = activity_prof::correlation_id;
+        correlation_id_ = amd::activity_prof::correlation_id;
       }
     }
 
@@ -120,12 +114,14 @@ class Event : public RuntimeObject {
     uint64_t correlation_id_;
     bool enabled_;        //!< Profiling enabled for the wave limiter
     bool marker_ts_;      //!< TS marker
+    bool batch_flush_ = true; //!< Command can flush the batch in direct dispatch mode
 
    void clear() {
       queued_ = 0ULL;
       submitted_ = 0ULL;
       start_ = 0ULL;
       end_ = 0ULL;
+      correlation_id_ = 0ULL;
      }
   } profilingInfo_;
 
@@ -153,10 +149,16 @@ class Event : public RuntimeObject {
   void EnableProfiling() {
     profilingInfo_.enabled_ = true;
     profilingInfo_.clear();
-    profilingInfo_.correlation_id_ = activity_prof::correlation_id;
+    profilingInfo_.correlation_id_ = amd::activity_prof::correlation_id;
   }
 
  public:
+  //! Use profiling info to force a tracking signal on command
+  void SetProfiling() {
+    EnableProfiling();
+    profilingInfo_.marker_ts_ = true;
+  }
+
   //! Return the context for this event.
   virtual const Context& context() const = 0;
 
@@ -253,13 +255,18 @@ union CopyMetadata {
  */
 class Command : public Event {
  private:
+  static SysmemPool<ComputeCommand> command_pool_;  //!< Pool of active commands
   HostQueue* queue_;               //!< The command queue this command is enqueue into
   Command* next_;                  //!< Next GPU command in the queue list
   Command* batch_head_ = nullptr;  //!< The head of the batch commands
   cl_command_type type_;           //!< This command's OpenCL type.
-  void* data_;
+  std::vector<void*> data_;
   const Event* waitingEvent_;  //!< Waiting event associated with the marker
 
+  bool packetCapturing_ = false;           //!< Flag to enable/disable graph gpu packet capture
+  uint8_t* gpuPacket_ = nullptr;     //!< GPU packet to capture, when graph capturing is enabled
+  address kernArgOffset_ = nullptr;  //!< KernelArg buffer to used when graph capturing is enabled
+  std::string* capturedKernelName_ = nullptr;  //!< Kenrnel under capture
  protected:
   bool cpu_wait_ = false;         //!< If true, then the command was issued for CPU/GPU sync
 
@@ -281,7 +288,6 @@ class Command : public Event {
         queue_(nullptr),
         next_(nullptr),
         type_(type),
-        data_(nullptr),
         waitingEvent_(nullptr),
         eventWaitList_(nullWaitList),
         commandWaitBits_(0) {}
@@ -297,6 +303,35 @@ class Command : public Event {
   }
 
  public:
+  //! Returns AQL buffer state
+  bool getPktCapturingState() const { return packetCapturing_; }
+
+  //! Sets AQL capture state, aql packet to capture and where to copy kernArgs
+  void setPktCapturingState(bool state, uint8_t* packet, address kernArgOffset,
+                            std::string* capturedKernelName) {
+    packetCapturing_ = state;
+    gpuPacket_ = packet;
+    kernArgOffset_ = kernArgOffset;
+    capturedKernelName_ = capturedKernelName;
+  }
+
+  //! Updates kernel name with the captured kernel name
+  void SetKernelName(const std::string& kernelName) {
+    if (capturedKernelName_ != nullptr) {
+      *capturedKernelName_ = kernelName;
+    }
+  }
+
+  //! returns the graph executable object command belongs to.
+  const uint8_t* getAqlPacket() const { return gpuPacket_; }
+
+  //! returns the graph executable object command belongs to.
+  const address getKernArgOffset() const { return kernArgOffset_; }
+
+  //! Overload new/delete for fast commands allocation/destruction
+  void* operator new(size_t size);
+  void operator delete(void* ptr);
+
   //! Return the queue this command is enqueued into.
   HostQueue* queue() const { return queue_; }
 
@@ -321,11 +356,9 @@ class Command : public Event {
   //! Return this command's OpenCL type.
   cl_command_type type() const { return type_; }
 
-  //! Return the opaque, device specific data for this command.
-  void* data() const { return data_; }
+  //! Return the opaque, device specific data vector for this command.
+  std::vector<void*>& data() { return data_; }
 
-  //! Set the opaque, device specific data for this command.
-  void setData(void* data) { data_ = data; }
 
   /*! \brief The execution engine for this command.
    *
@@ -901,6 +934,8 @@ class CopyMemoryCommand : public TwoMemoryArgsCommand {
   const BufferRect& dstRect() const { return dstRect_; }
   //! Return the copy MetaData
   amd::CopyMetadata copyMetadata() const { return copyMetadata_; }
+  //! Updates copy MetaData
+  void SetCopyMetadata(amd::CopyMetadata copyMetadata) { copyMetadata_ = copyMetadata; }
   //! Updates the host memory to read from
   void setSource(Memory& srcMemory) { memory1_ = &srcMemory; }
   //! Updates the memory object to write to.
@@ -1078,10 +1113,6 @@ class NDRangeKernelCommand : public Command {
   uint32_t firstDevice_;    //!< Device index of the first device in the gridc
   uint32_t numWorkgroups_;  //!< Total number of workgroups in the current launch
 
-  bool capturing_ = false;           //!< Flag to enable/disable graph gpu packet capture
-  uint8_t* gpuPacket_ = nullptr;     //!< GPU packet to capture, when graph capturing is enabled
-  address kernArgOffset_ = nullptr;  //!< KernelArg buffer to used when graph capturing is enabled
-
  public:
   enum {
     CooperativeGroups = 0x01,
@@ -1089,22 +1120,6 @@ class NDRangeKernelCommand : public Command {
     AnyOrderLaunch = 0x04,
   };
 
-  //! Returns AQL buffer state
-  bool getCapturingState() const { return capturing_; }
-
-  //! Sets AQL capture state, aql packet to capture and where to copy kernArgs
-  void setCapturingState(bool state, uint8_t* packet, address kernArgOffset) {
-    capturing_ = state;
-    gpuPacket_ = packet;
-    kernArgOffset_ = kernArgOffset;
-  }
-
-  //! returns the graph executable object command belongs to.
-  const uint8_t* getAqlPacket() const { return gpuPacket_; }
-
-  //! returns the graph executable object command belongs to.
-  const address getKernArgOffset() const { return kernArgOffset_; }
-
   //! Construct an ExecuteKernel command
   NDRangeKernelCommand(HostQueue& queue, const EventWaitList& eventWaitList, Kernel& kernel,
                        const NDRangeContainer& sizes, uint32_t sharedMemBytes = 0,
@@ -1180,7 +1195,11 @@ class NDRangeKernelCommand : public Command {
     numWorkgroups_ = numWorkgroups;
   }
 
+  // Capture kernel parameters and validate
   int32_t captureAndValidate();
+
+  // Allocate, capture and set kernel parameters
+  int32_t AllocCaptureSetValidate(void** kernelParams, address kernArgs);
 };
 
 class NativeFnCommand : public Command {
@@ -1254,8 +1273,6 @@ class Marker : public Command {
 
 class AccumulateCommand : public Command {
  private:
-  uint8_t* lastPacket_;
-
   //! Kernel names and timestamps list for activity profiling
   std::vector<std::string> kernelNames_;
   std::vector<std::pair<uint64_t, uint64_t>> tsList_;
@@ -1263,26 +1280,18 @@ class AccumulateCommand : public Command {
  public:
   //! Create a new Marker
   AccumulateCommand(HostQueue& queue, const EventWaitList& eventWaitList = nullWaitList,
-         const Event* waitingEvent = nullptr, uint8_t* lastPacket = nullptr)
-      : Command(queue, CL_COMMAND_TASK, eventWaitList, 0, waitingEvent),
-        lastPacket_(lastPacket)
+         const Event* waitingEvent = nullptr)
+      : Command(queue, CL_COMMAND_TASK, eventWaitList, 0, waitingEvent)
       {}
-  //! Return last packet
-  uint8_t* getLastPacket() const { return lastPacket_; }
 
   //! Add kernel name to the list if available
   void addKernelName(const std::string& kernelName) {
-    if (activity_prof::IsEnabled(OP_ID_DISPATCH)) {
-      // "^" is to indicate kernel is captured at instantiate
-      kernelNames_.push_back("^  " + kernelName);
-    }
+    kernelNames_.push_back(kernelName);
   }
 
   //! Add kernel timestamp to the list if available
   void addTimestamps(uint64_t startTs, uint64_t endTs) {
-    if (activity_prof::IsEnabled(OP_ID_DISPATCH)) {
-      tsList_.push_back(std::make_pair(startTs, endTs));
-    }
+    tsList_.push_back(std::make_pair(startTs, endTs));
   }
 
   //! Return the kernel names
@@ -1794,6 +1803,39 @@ public:
   const void* ptr() const { return ptr_; }
 };
 
+//! Union used in memory suballocator, must be updated with the new commands
+union ComputeCommand {
+  ReadMemoryCommand             cmd0;
+  WriteMemoryCommand            cmd1;
+  FillMemoryCommand             cmd2;
+  CopyMemoryCommand             cmd3;
+  MapMemoryCommand              cmd4;
+  UnmapMemoryCommand            cmd5;
+  MigrateMemObjectsCommand      cmd6;
+  NDRangeKernelCommand          cmd7;
+  NativeFnCommand               cmd8;
+  ExternalSemaphoreCmd          cmd9;
+  Marker                        cmd10;
+  AccumulateCommand             cmd11;
+  AcquireExtObjectsCommand      cmd13;
+  ReleaseExtObjectsCommand      cmd14;
+  PerfCounterCommand            cmd15;
+  ThreadTraceMemObjectsCommand  cmd16;
+  ThreadTraceCommand            cmd17;
+  SignalCommand                 cmd18;
+  MakeBuffersResidentCommand    cmd19;
+  SvmFreeMemoryCommand          cmd20;
+  SvmCopyMemoryCommand          cmd21;
+  SvmFillMemoryCommand          cmd22;
+  SvmMapMemoryCommand           cmd23;
+  SvmUnmapMemoryCommand         cmd24;
+  CopyMemoryP2PCommand          cmd25;
+  SvmPrefetchAsyncCommand       cmd26;
+  VirtualMapCommand             cmd27;
+  ComputeCommand() {}
+  ~ComputeCommand() {}
+};
+
 /*! @}
  *  @}
  */
