@@ -28,11 +28,11 @@
 
 #include <atomic>
 
+namespace amd {
+
 // Stores the no. of memory allocations
 std::atomic<uint32_t> numAllocs = {0};
 
-namespace amd {
-
 bool BufferRect::create(const size_t* bufferOrigin, const size_t* region, size_t bufferRowPitch,
                         size_t bufferSlicePitch) {
   bool valid = false;
@@ -114,7 +114,7 @@ Memory::Memory(Memory& parent, Flags flags, size_t origin, size_t size, Type typ
       flags_(flags),
       version_(parent.getVersion()),
       lastWriter_(parent.getLastWriter()),
-      interopObj_(nullptr),
+      interopObj_(amd::IS_HIP ? nullptr : parent.getInteropObj()),
       vDev_(NULL),
       mapCount_(0),
       svmHostAddress_(parent.getSvmPtr()),
@@ -320,10 +320,7 @@ bool Memory::create(void* initFrom, bool sysMemAlloc, bool skipAlloc, bool force
       }
     }
   }
-  // Add a VA range into VA range map
-  if (getMemFlags() & CL_MEM_VA_RANGE_AMD) {
-    amd::MemObjMap::AddVirtualMemObj(getSvmPtr(), this);
-  }
+
   // Store the unique id for each memory allocation
   uniqueId_ = ++numAllocs;
   return true;
@@ -419,21 +416,21 @@ device::Memory* Memory::getDeviceMemory(const Device& dev, bool alloc) {
 Memory::~Memory() {
   // For_each destructor callback:
   DestructorCallBackEntry* entry;
-  for (entry = destructorCallbacks_; entry != NULL; entry = entry->next_) {
+  for (entry = destructorCallbacks_; entry != nullptr; entry = entry->next_) {
     // invoke the callback function.
     entry->callback_(const_cast<cl_mem>(as_cl(this)), entry->data_);
   }
 
   // Release the parent.
-  if (NULL != parent_) {
+  if (parent_ != nullptr) {
     // Update cache if runtime destroys a subbuffer
-    if (NULL != parent_->getHostMem() && (vDev_ == NULL)) {
+    if (parent_->getHostMem() != nullptr && (vDev_ == nullptr)) {
       cacheWriteBack(nullptr);
     }
     parent_->removeSubBuffer(this);
   }
 
-  if (NULL != deviceMemories_) {
+  if (deviceMemories_ != nullptr) {
     // Destroy all device memory objects
     for (uint i = 0; i < numDevices_; ++i) {
       delete deviceMemories_[i].value_;
@@ -447,19 +444,22 @@ Memory::~Memory() {
 
   // Destroy the destructor callback entries
   DestructorCallBackEntry* callback = destructorCallbacks_;
-  while (callback != NULL) {
+  while (callback != nullptr) {
     DestructorCallBackEntry* next = callback->next_;
     delete callback;
     callback = next;
   }
 
   // Make sure runtime destroys the parent only after subbuffer destruction
-  if (NULL != parent_) {
+  if (parent_ != nullptr) {
     parent_->release();
   }
   hostMemRef_.deallocateMemory(context_());
-  if (getMemFlags() & CL_MEM_VA_RANGE_AMD) {
-    amd::MemObjMap::RemoveVirtualMemObj(getSvmPtr());
+  if (parent_ == nullptr && (getMemFlags() & CL_MEM_VA_RANGE_AMD)) {
+    // The mapping may manually be removed prior to objects destruction
+    if (amd::MemObjMap::FindVirtualMemObj(getSvmPtr())) {
+      amd::MemObjMap::RemoveVirtualMemObj(getSvmPtr());
+    }
     // If runtime executes graph mempool with VM, then VA can be mapped in space
     // for graph validation logic during execution. And the reason it's not unmaped
     // in graph itself because the app can have a graph without a free node
@@ -543,6 +543,11 @@ void Memory::uncommitSvmMemory() {
   }
 }
 
+Device* Memory::GetDeviceById() {
+  size_t device_idx = (userData_.deviceId < getContext().devices().size()) ? userData_.deviceId : 0;
+  return getContext().devices()[device_idx];
+}
+
 void Buffer::initDeviceMemory() {
   deviceMemories_ = reinterpret_cast<DeviceMemory*>(reinterpret_cast<char*>(this) + sizeof(Buffer));
   memset(deviceMemories_, 0, NumDevicesWithP2P() * sizeof(DeviceMemory));
@@ -1524,7 +1529,6 @@ bool SvmBuffer::Contains(uintptr_t ptr) {
 // The allocation flags are ignored for now.
 void* SvmBuffer::malloc(Context& context, cl_svm_mem_flags flags, size_t size, size_t alignment,
                         const amd::Device* curDev) {
-  bool atomics = (flags & CL_MEM_SVM_ATOMICS) != 0;
   void* ret = context.svmAlloc(size, alignment, flags, curDev);
   if (ret == nullptr) {
     LogError("Unable to allocate aligned memory");
