@@ -183,7 +183,7 @@ release(uint, HIP_HIDDEN_FREE_MEM, 0,                                         \
         "Reserve free mem reporting in Mb"                                    \
         "0 = Disable")                                                        \
 release(size_t, GPU_FORCE_BLIT_COPY_SIZE, 0,                                  \
-        "Size in KB of the threshold below which to force blit instead for sdma") \
+        "Use Blit until this size(in KB) for copies")                         \
 release(uint, ROC_ACTIVE_WAIT_TIMEOUT, 0,                                     \
         "Forces active wait of GPU interrup for the timeout(us)")             \
 release(bool, ROC_ENABLE_LARGE_BAR, true,                                     \
@@ -194,11 +194,11 @@ release(bool, ROC_SYSTEM_SCOPE_SIGNAL, true,                                  \
         "Enable system scope for signals (uses interrupts).")                 \
 release(bool, GPU_FORCE_QUEUE_PROFILING, false,                               \
         "Force command queue profiling by default")                           \
-release(bool, HIP_MEM_POOL_SUPPORT, false,                                    \
+release(bool, HIP_MEM_POOL_SUPPORT, true,                                     \
         "Enables memory pool support in HIP")                                 \
-release(bool, HIP_MEM_POOL_USE_VM, IS_WINDOWS,                                \
+release(bool, HIP_MEM_POOL_USE_VM, true,                                      \
         "Enables memory pool support in HIP")                                 \
-release(bool, PAL_HIP_IPC_FLAG, false,                                        \
+release(bool, PAL_HIP_IPC_FLAG, true,                                         \
         "Enable interprocess flag for device allocation in PAL HIP")          \
 release(uint, PAL_FORCE_ASIC_REVISION, 0,                                     \
         "Force a specific asic revision for all devices")                     \
@@ -219,10 +219,12 @@ release(uint, ROC_P2P_SDMA_SIZE, 1024,                                        \
         "The minimum size in KB for P2P transfer with SDMA")                  \
 release(uint, ROC_AQL_QUEUE_SIZE, 16384,                                      \
         "AQL queue size in AQL packets")                                      \
-release(uint, ROC_SIGNAL_POOL_SIZE, 32,                                       \
+release(uint, ROC_SIGNAL_POOL_SIZE, 64,                                       \
         "Initial size of HSA signal pool")                                    \
 release(uint, DEBUG_CLR_LIMIT_BLIT_WG, 16,                                    \
         "Limit the number of workgroups in blit operations")                  \
+release(bool, DEBUG_CLR_BLIT_KERNARG_OPT, false,                              \
+        "Enable blit kernel arguments optimization")                          \
 release(bool, ROC_SKIP_KERNEL_ARG_COPY, false,                                \
         "If true, then runtime can skip kernel arg copy")                     \
 release(bool, GPU_STREAMOPS_CP_WAIT, false,                                   \
@@ -235,7 +237,7 @@ release(size_t, HIP_INITIAL_DM_SIZE, 8 * Mi,                                  \
         "Set initial heap size for device malloc.")                           \
 release(bool, HIP_FORCE_DEV_KERNARG, true,                                    \
          "Force device mem for kernel args.")                                 \
-release(bool, DEBUG_CLR_GRAPH_PACKET_CAPTURE, false,                          \
+release(bool, DEBUG_CLR_GRAPH_PACKET_CAPTURE, true,                           \
          "Enable/Disable graph packet capturing")                             \
 release(bool, GPU_DEBUG_ENABLE, false,                                        \
         "Enables collection of extra info for debugger at some perf cost")    \
@@ -245,6 +247,24 @@ release(cstring, HIPRTC_LINK_OPTIONS_APPEND, "",                              \
         "Set link options needed for hiprtc compilation")                     \
 release(bool, HIP_VMEM_MANAGE_SUPPORT, true,                                  \
         "Virtual Memory Management Support")                                  \
+release(bool, DEBUG_HIP_GRAPH_DOT_PRINT, false,                               \
+         "Enable/Disable graph debug dot print dump")                         \
+release(bool, DEBUG_HIP_FORCE_ASYNC_QUEUE, false,                             \
+        "Forces grpahs into async queue mode. DEBUG_HIP_FORCE_GRAPH_QUEUES must be 1") \
+release(uint, DEBUG_HIP_FORCE_GRAPH_QUEUES, 4,                                \
+        "Forces the number of streams for the graph parallel execution")      \
+release(bool, HIP_ALWAYS_USE_NEW_COMGR_UNBUNDLING_ACTION, false,              \
+        "Force to always use new comgr unbundling action")                    \
+release(uint, DEBUG_HIP_BLOCK_SYNC, 50,                                       \
+        "Blocks synchronization on CPU until the callback processing is done")\
+release(uint, DEBUG_CLR_MAX_BATCH_SIZE, 1000,                                 \
+        "Forces the callback to clean-up CPU submission queue")               \
+release(bool, DEBUG_CLR_SYSMEM_POOL, false,                                   \
+        "Use sysmem pool implementation in runtime for amd commands")         \
+release(bool, DEBUG_HIP_KERNARG_COPY_OPT, true,                               \
+         "Enable/Disable multiple kern arg copies")                           \
+release(bool, DEBUG_CLR_KERNARG_HDP_FLUSH_WA, false,                          \
+        "Toggle kernel arg copy workaround")                                  \
 
 namespace amd {
 
@@ -320,7 +340,12 @@ struct Flag {
 #ifdef _WIN32
 # define EXPORT_FLAG extern "C" __declspec(dllexport)
 #else // !_WIN32
+#ifdef BUILD_STATIC_LIBS
+# define EXPORT_FLAG extern
+#else
 # define EXPORT_FLAG extern "C"
+#endif
+namespace amd::flags {
 #endif // !_WIN32
 
 #define DECLARE_RELEASE_FLAG(type, name, value, help) EXPORT_FLAG type name;
@@ -334,5 +359,8 @@ RUNTIME_FLAGS(DECLARE_DEBUG_FLAG, DECLARE_RELEASE_FLAG, DECLARE_DEBUG_FLAG);
 
 #undef DECLARE_DEBUG_FLAG
 #undef DECLARE_RELEASE_FLAG
-
+#ifndef _WIN32
+}
+using namespace amd::flags;
+#endif // !_WIN32
 #endif /*FLAGS_HPP_*/
